@@ -22,7 +22,9 @@ import unittest
 
 from slapos.runner.utils import (getProfilePath, getSession, isInstanceRunning,
                                  isSoftwareRunning, startProxy,
-                                 isSoftwareReleaseReady)
+                                 isSoftwareReleaseReady,
+                                 runSlapgridUntilSuccess,
+                                 getBuildAndRunParams, saveBuildAndRunParams)
 from slapos.runner.process import killRunningProcess, isRunning
 from slapos.runner import views
 import slapos.slap
@@ -94,12 +96,25 @@ class SlaprunnerTestCase(unittest.TestCase):
       software_profile='software.cfg',
       SECRET_KEY="123456",
       PERMANENT_SESSION_LIFETIME=datetime.timedelta(days=31),
+      auto_deploy = True,
+      autorun = False,
+      instance_monitoring_url = 'https://[' + config.ipv6_address + ']:9684',
     )
     self.app = views.app.test_client()
     self.app.config = views.app.config
     #Create password recover code
     with open(os.path.join(views.app.config['etc_dir'], '.rcode'), 'w') as rpwd:
       rpwd.write(self.rcode)
+    #Create config.json
+    json_file = os.path.join(views.app.config['etc_dir'], 'config.json')
+    if not os.path.exists(json_file):
+      params = {
+        'run_instance' : True,
+        'run_software' : True,
+        'max_run_instance' : 3,
+        'max_run_software' : 2
+      }
+      open(json_file, "w").write(json.dumps(params))
 
   def tearDown(self):
     """Remove all test data"""
@@ -322,10 +337,9 @@ class SlaprunnerTestCase(unittest.TestCase):
 
     # Compile software and wait until slapgrid ends
     # this is supposed to use current SR
-    response = loadJson(self.app.post('/runSoftwareProfile',
-                                      data=dict(),
-                                      follow_redirects=True))
-    self.assertTrue(response['result'])
+    while self.app.get('/isSRReady').data == "2":
+      time.sleep(2)
+    self.assertEqual(self.app.get('/isSRReady').data, "1")
     self.assertTrue(os.path.exists(self.app.config['software_root']))
     self.assertTrue(os.path.exists(self.app.config['software_log']))
     assert "test-application" in open(self.app.config['software_log']).read()
@@ -387,12 +401,16 @@ class SlaprunnerTestCase(unittest.TestCase):
     response = loadJson(self.app.post('/runSoftwareProfile',
                                       data=dict(),
                                       follow_redirects=True))
-    self.assertTrue(response['result'])
+    while self.app.get('/isSRReady').data == "2":
+      time.sleep(2)
+    self.assertEqual(self.app.get('/isSRReady').data, "1")
     #run instance profile
     response = loadJson(self.app.post('/runInstanceProfile',
                                       data=dict(),
                                       follow_redirects=True))
     self.assertTrue(response['result'])
+    # lets some time to the Instance to be deployed
+    time.sleep(5)
     #Check that all partitions has been created
     assert "create-file" in open(self.app.config['instance_log']).read()
     instanceDir = os.listdir(self.app.config['instance_root'])
@@ -438,6 +456,69 @@ class SlaprunnerTestCase(unittest.TestCase):
     self.assertEqual(response, "1")
     assert completed_text in open(completed_path).read()
 
+  def test_maximumRunOfSlapgrid(self):
+    """Scenario 8: runSlapgridUntilSucces run until a defined maximum of time
+    slapgrid-sr and slapgrid-cp if it fails. It can also run only one or both
+    of them if it is defined so
+    We directly calls runSlapgridUntilSuccess, because we want
+    to test the return code of the function"""
+    # Installs a wrong buildout which will fail
+    MAX_RUN_SOFTWARE = getBuildAndRunParams(self.app.config)['max_run_software']
+    MAX_RUN_INSTANCE = getBuildAndRunParams(self.app.config)['max_run_instance']
+    self.test_createSR()
+    newSoftware = self.getCurrentSR()
+    softwareRelease = "[buildout]\n\nparts =\n  test-application\n"
+    softwareRelease += "#Test download git web repos éè@: utf-8 caracters\n"
+    softwareRelease += "[test-application]\nrecipe = slapos.cookbook:mkdirectory\n"
+    softwareRelease += "test = /root/test\n"
+    response = loadJson(self.app.post('/saveFileContent',
+                                      data=dict(file=newSoftware,
+                                                content=softwareRelease),
+                                      follow_redirects=True))
+    response = runSlapgridUntilSuccess(self.app.config, 'software')
+    self.assertEqual(response, MAX_RUN_SOFTWARE)
+    # clean folders for other tests
+    workdir = os.path.join(self.app.config['runner_workdir'], 'project')
+    git_repo = os.path.join(workdir, 'slapos')
+    if os.path.exists(git_repo):
+      shutil.rmtree(git_repo)
+    # Installs a software which deploys, but fails while instanciating
+    # preparation
+    base = os.path.join(self.app.config['workspace'], 'slapos')
+    software = os.path.join(base, 'software')
+    testSoftware = os.path.join(software, 'slaprunner-test')
+    if not os.path.exists(testSoftware):
+      os.makedirs(testSoftware)
+    software_cfg = os.path.join(testSoftware, 'software.cfg')
+    instance_cfg = os.path.join(testSoftware, 'instance.cfg')
+    # software.cfg
+    softwareRelease = "[buildout]\n\nparts =\n  failing-template\n\n"
+    softwareRelease += "[failing-template]\nrecipe = hexagonit.recipe.download\n"
+    softwareRelease += "url = %s\n" % (instance_cfg)
+    softwareRelease += "destination = ${buildout:directory}\n"
+    softwareRelease += "download-only = true\n"
+    open(software_cfg, 'w+').write(softwareRelease)
+    # instance.cfg
+    content = "[buildout]\n\nparts =\n fail\n"
+    content += "[fail]\nrecipe=plone.recipe.command\n"
+    content += "command = exit 1"
+    open(instance_cfg, 'w+').write(content)
+    project = open(os.path.join(self.app.config['etc_dir'],
+                  '.project'), "w")
+    project.write(self.software + 'slaprunner-test')
+    project.close()
+    # Build and Run
+    parameters = getBuildAndRunParams(self.app.config)
+    parameters['run_instance'] = False
+    saveBuildAndRunParams(self.app.config, parameters)
+    response = runSlapgridUntilSuccess(self.app.config, 'software')
+    self.assertEqual(response, 1)
+    parameters['run_instance'] = True
+    saveBuildAndRunParams(self.app.config, parameters)
+    response = runSlapgridUntilSuccess(self.app.config, 'software')
+    self.assertEqual(response, (1, MAX_RUN_INSTANCE))
+
+
 def main():
   # Empty parser for now - so that erp5testnode is happy when doing --help
   parser = argparse.ArgumentParser()
