@@ -2,6 +2,7 @@
 # vim: set et sts=2:
 # pylint: disable-msg=W0311,C0301,C0103,C0111,W0141,W0142
 
+import json
 import logging
 import md5
 import multiprocessing
@@ -36,6 +37,19 @@ html_escape_table = {
   "<": "&lt;",
 }
 
+def getBuildAndRunParams(config):
+  json_file = os.path.join(config['etc_dir'], 'config.json')
+  json_params = json.load(open(json_file))
+  return json_params
+
+
+def saveBuildAndRunParams(config, params):
+  """XXX-Nico parameters have to be correct.
+  Works like that because this function do not care
+  about how you got the parameters"""
+  json_file = os.path.join(config['etc_dir'], 'config.json')
+  open(json_file, "w").write(json.dumps(params))
+
 
 def html_escape(text):
   """Produce entities within text."""
@@ -98,6 +112,16 @@ def saveSession(config, account):
     return str(e)
 
 
+def createNewUser(config, name, passwd):
+  htpasswdfile = os.path.join(config['etc_dir'], '.htpasswd')
+  if os.path.exists(htpasswdfile):
+    htpasswd = HtpasswdFile(htpasswdfile)
+    htpasswd.update(name, passwd)
+    htpasswd.save()
+    return True
+  return False
+
+
 def getCurrentSoftwareReleaseProfile(config):
   """
   Returns used Software Release profile as a string.
@@ -235,7 +259,7 @@ def isSoftwareRunning(config=None):
   return isRunning('slapgrid-sr')
 
 
-def runSoftwareWithLock(config):
+def runSoftwareWithLock(config, lock=True):
   """
     Use Slapgrid to compile current Software Release and wait until
     compilation is done
@@ -251,19 +275,17 @@ def runSoftwareWithLock(config):
   logfile = open(config['software_log'], 'w')
   if not updateProxy(config):
     return False
-  # Accelerate compilation by setting make -jX
-  # XXX-Marco can have issues with implicit dependencies or recursive makefiles. should be configurable.
-  environment = os.environ.copy()
-  environment['MAKEFLAGS'] = '-j%r' % multiprocessing.cpu_count()
   slapgrid = Popen([config['slapgrid_sr'], '-vc',
                     '--pidfile', slapgrid_pid,
                     config['configuration_file_path'], '--now', '--develop'],
-                   stdout=logfile, env=environment,
-                   name='slapgrid-sr')
-  slapgrid.wait()
-  #Saves the current compile software for re-use
-  config_SR_folder(config)
-  return True
+                   stdout=logfile, name='slapgrid-sr')
+  if lock:
+    slapgrid.wait()
+    #Saves the current compile software for re-use
+    config_SR_folder(config)
+    return ( True if slapgrid.returncode == 0 else False )
+  else:
+    return False
 
 
 def config_SR_folder(config):
@@ -328,7 +350,7 @@ def isInstanceRunning(config=None):
   return isRunning('slapgrid-cp')
 
 
-def runInstanceWithLock(config):
+def runInstanceWithLock(config, lock=True):
   """
     Use Slapgrid to deploy current Software Release and wait until
     deployment is done.
@@ -345,8 +367,11 @@ def runInstanceWithLock(config):
                     '--pidfile', slapgrid_pid,
                     config['configuration_file_path'], '--now'],
                     stdout=logfile, name='slapgrid-cp')
-  slapgrid.wait()
-  return True
+  if lock:
+    slapgrid.wait()
+    return ( True if slapgrid.returncode == 0 else False )
+  else:
+    return False
 
 
 def getProfilePath(projectDir, profile):
@@ -450,94 +475,22 @@ def svcStartStopProcess(config, process, action):
                 cmd[action], process]).communicate()[0]
 
 
-def getFolderContent(config, folder):
-  """
-  Read all file and folder into specified directory
-
-  Args:
-    config: Slaprunner configuration.
-    folder: the directory to read.
-
-  Returns:
-    Html formatted string or error message when fail.
-  """
-  r = ['<ul class="jqueryFileTree" style="display: none;">']
-  try:
-    folder = str(folder)
-    r = ['<ul class="jqueryFileTree" style="display: none;">']
-    d = urllib.unquote(folder)
-    realdir = realpath(config, d)
-    if realdir:
-      ldir = sorted(os.listdir(realdir), key=str.lower)
-    else:
-      r.append('Could not load directory: Permission denied')
-      ldir = []
-
-    for f in ldir:
-      if f.startswith('.'):  # do not displays this file/folder
-        continue
-      ff = os.path.join(d, f)
-      if os.path.isdir(os.path.join(realdir, f)):
-        r.append('<li class="directory collapsed"><a href="#%s" rel="%s/">%s</a></li>' % (ff, ff, f))
-      else:
-        e = os.path.splitext(f)[1][1:]  # get .ext and remove dot
-        r.append('<li class="file ext_%s"><a href="#%s" rel="%s">%s</a></li>' % (e, ff, ff, f))
-    r.append('</ul>')
-  except Exception as e:
-    r.append('Could not load directory: %s' % str(e))
-  r.append('</ul>')
-  return jsonify(result=''.join(r))
-
-
-def getFolder(config, folder):
-  """
-  Read list of folder for the specified directory
-
-  Args:
-    config: Slaprunner configuration.
-    folder: the directory to read.
-
-  Returns:
-    Html formatted string or error message when fail.
-  """
-  r = ['<ul class="jqueryFileTree" style="display: none;">']
-  try:
-    folder = str(folder)
-    r = ['<ul class="jqueryFileTree" style="display: none;">']
-    d = urllib.unquote(folder)
-    realdir = realpath(config, d)
-    if not realdir:
-      r.append('Could not load directory: Permission denied')
-      ldir = []
-    else:
-      ldir = sorted(os.listdir(realdir), key=str.lower)
-    for f in ldir:
-      if f.startswith('.'):  # do not display this file/folder
-        continue
-      ff = os.path.join(d, f)
-      if os.path.isdir(os.path.join(realdir, f)):
-        r.append('<li class="directory collapsed"><a href="#%s" rel="%s/">%s</a></li>' % (ff, ff, f))
-    r.append('</ul>')
-  except Exception as e:
-    r.append('Could not load directory: %s' % str(e))
-  r.append('</ul>')
-  return jsonify(result=''.join(r))
-
-
-def getProjectList(folder):
-  """Return the list of projet (folder) into the workspace
+def listFolder(config, path):
+  """Return the list of folder into path
 
   Agrs:
-    folder: path of the workspace
+    path: path of the directory to list
   Returns:
     a list that contains each folder name.
   """
-  project = []
-  project_list = sorted(os.listdir(folder), key=str.lower)
-  for elt in project_list:
-    if os.path.isdir(os.path.join(folder, elt)):
-      project.append(elt)
-  return project
+  folderList = []
+  folder = realpath(config, path)
+  if folder:
+    path_list = sorted(os.listdir(folder), key=str.lower)
+    for elt in path_list:
+      if os.path.isdir(os.path.join(folder, elt)):
+        folderList.append(elt)
+  return folderList
 
 
 def configNewSR(config, projectpath):
@@ -771,6 +724,7 @@ def realpath(config, path, check_exist=True):
     'software_root': config['software_root'],
     'instance_root': config['instance_root'],
     'workspace': config['workspace'],
+    'runner_workdir': config['runner_workdir'],
     'software_link': config['software_link']
   }
   if key not in allow_list:
@@ -827,6 +781,8 @@ def isSoftwareReleaseReady(config):
   config_SR_folder(config)
   if os.path.exists(os.path.join(config['runner_workdir'],
       'softwareLink', software_name, '.completed')):
+    if config['autorun'] in TRUE_VALUES:
+      runSlapgridUntilSuccess(config, 'instance')
     return "1"
   else:
     if isSoftwareRunning(config):
@@ -834,6 +790,10 @@ def isSoftwareReleaseReady(config):
     elif config['auto_deploy'] in TRUE_VALUES:
       configNewSR(config, path)
       runSoftwareWithLock(config)
+      config_SR_folder(config)
+      time.sleep(15)
+      if config['autorun'] in TRUE_VALUES:
+        runSlapgridUntilSuccess(config, 'instance')
       return "2"
     else:
       return "0"
@@ -856,6 +816,43 @@ def buildAndRun(config):
   runInstanceWithLock(config)
 
 
+def runSlapgridUntilSuccess(config, step):
+  """Run slapgrid-sr or slapgrid-cp several times,
+  in the maximum of the constant MAX_RUN_~~~~"""
+  params = getBuildAndRunParams(config)
+  if step == "instance":
+    max_tries = (params['max_run_instance'] if params['run_instance'] else 0)
+    runSlapgridWithLock = runInstanceWithLock
+  elif step == "software":
+    max_tries = (params['max_run_software'] if params['run_software'] else 0)
+    runSlapgridWithLock = runSoftwareWithLock
+  else:
+    return -1
+  counter_file = os.path.join(config['runner_workdir'], '.turn-left')
+  open(counter_file, 'w+').write(str(max_tries))
+  counter = max_tries
+  slapgrid = True
+  # XXX-Nico runSoftwareWithLock can return 0 or False (0==False)
+  while counter > 0:
+    counter -= 1
+    slapgrid = runSlapgridWithLock(config)
+    if slapgrid:
+      break
+    times_left = int(open(counter_file).read()) - 1
+    if times_left > 0 :
+      open(counter_file, 'w+').write(str(times_left))
+      counter = times_left
+    else :
+      counter = 0
+  max_tries -= counter
+  # run instance only if we are deploying the software release,
+  # if it is defined so, and sr is correctly deployed
+  if step == "software" and params['run_instance'] and slapgrid:
+    return (max_tries, runSlapgridUntilSuccess(config, "instance"))
+  else:
+    return max_tries
+
+
 def setupDefaultSR(config):
   """If a default_sr is in the parameters,
   and no SR is deployed yet, setup it
