@@ -3,18 +3,20 @@
 # pylint: disable-msg=W0311,C0301,C0103,C0111
 
 
+import json
 import os
 import shutil
+import thread
 import urllib
 
-from flaskext.auth import logout
 from flask import (Flask, request, redirect, url_for, render_template,
                    g, flash, jsonify, session, abort, send_file)
 
 from slapos.runner.process import killRunningProcess
-from slapos.runner.utils import (checkSoftwareFolder, configNewSR, getFolder,
-                                 getFolderContent, getProfilePath,
-                                 getProjectList, getProjectTitle, getSession,
+from slapos.runner.utils import (checkSoftwareFolder, configNewSR,
+                                 createNewUser, getProfilePath,
+                                 listFolder, getBuildAndRunParams,
+                                 getProjectTitle, getSession,
                                  getSlapStatus, getSvcStatus,
                                  getSvcTailProcess, isInstanceRunning,
                                  isSoftwareRunning, isSoftwareReleaseReady, isText,
@@ -22,7 +24,8 @@ from slapos.runner.utils import (checkSoftwareFolder, configNewSR, getFolder,
                                  readFileFrom, readParameters, realpath,
                                  removeInstanceRoot, removeProxyDb,
                                  removeSoftwareByName, runInstanceWithLock,
-                                 runSoftwareWithLock, saveSession,
+                                 runSoftwareWithLock, runSlapgridUntilSuccess,
+                                 saveSession, saveBuildAndRunParams,
                                  svcStartStopProcess, svcStopAll, tail,
                                  updateInstanceParameter)
 
@@ -58,6 +61,7 @@ def before_request():
     if request.path != "/setAccount" and request.path != "/configAccount":
       return redirect(url_for('setAccount'))
 
+  g.instance_monitoring_url =  app.config['instance_monitoring_url']
 
 # general views
 def home():
@@ -85,19 +89,15 @@ def setAccount():
 def myAccount():
   account = getSession(app.config)
   return render_template('account.html', username=account[0],
-          email=account[2], name=account[3].decode('utf-8'))
+          email=account[2], name=account[3].decode('utf-8'),
+          params=getBuildAndRunParams(app.config))
 
 
-@app.route("/dologout")
-def dologout():
-  _ = logout()
-  return redirect(url_for('login'))
-
-
-def configRepo():
+def manageRepository():
   public_key = open(app.config['public_key']).read()
   account = getSession(app.config)
-  return render_template('cloneRepository.html', workDir='workspace',
+  return render_template('manageRepository.html', workDir='workspace',
+            project=listFolder(app.config, 'workspace'),
             public_key=public_key, name=account[3].decode('utf-8'),
             email=account[2])
 
@@ -115,7 +115,7 @@ def editSoftwareProfile():
   if profile == "":
     flash('Error: can not open profile, please select your project first')
   return render_template('updateSoftwareProfile.html', workDir='workspace',
-      profile=profile, projectList=getProjectList(app.config['workspace']))
+      profile=profile, projectList=listFolder(app.config, 'workspace'))
 
 
 def inspectSoftware():
@@ -137,19 +137,8 @@ def removeSoftware():
 
 
 def runSoftwareProfile():
-  if runSoftwareWithLock(app.config):
-    return jsonify(result=True)
-  else:
-    return jsonify(result=False)
-
-
-def viewSoftwareLog():
-  if os.path.exists(app.config['software_log']):
-    result = tail(open(app.config['software_log']), lines=1500)
-  else:
-    result = 'Not found yet'
-  return render_template('viewLog.html', type='software',
-      result=result.encode("utf-8"))
+  thread.start_new_thread(runSlapgridUntilSuccess, (app.config, "software"))
+  return jsonify(result=True)
 
 
 # instance views
@@ -158,7 +147,7 @@ def editInstanceProfile():
   if profile == "":
     flash('Error: can not open instance profile for this Software Release')
   return render_template('updateInstanceProfile.html', workDir='workspace',
-      profile=profile, projectList=getProjectList(app.config['workspace']))
+      profile=profile, projectList=listFolder(app.config, 'workspace'))
 
 
 # get status of all computer partitions and process state
@@ -210,19 +199,33 @@ def removeInstance():
 def runInstanceProfile():
   if not os.path.exists(app.config['instance_root']):
     os.mkdir(app.config['instance_root'])
-  if runInstanceWithLock(app.config):
-    return jsonify(result=True)
-  else:
-    return jsonify(result=False)
+  thread.start_new_thread(runSlapgridUntilSuccess, (app.config, "instance"))
+  return jsonify(result=True)
 
 
-def viewInstanceLog():
-  if os.path.exists(app.config['instance_log']):
-    result = open(app.config['instance_log']).read()
+def viewLog():
+  return render_template('viewLog.html')
+
+def getFileLog():
+  logfile = request.form.get('filename', '').encode('utf-8')
+  if logfile == "instance.log":
+    file_path = app.config['instance_log']
+  elif logfile == "software.log":
+    file_path = app.config['software_log']
   else:
-    result = 'Not found yet'
-  return render_template('viewLog.html', type='instance',
-      result=result.encode("utf-8"))
+    file_path = realpath(app.config, logfile)
+  try:
+    if not isText(file_path):
+      return jsonify(code=0,
+            result="Can not open binary file, please select a text file!")
+    if 'truncate' in request.form:
+      content = tail(open(file_path), int(request.form['truncate']))
+      return jsonify(code=1, result=content)
+    else:
+      with open(file_path) as f:
+        return jsonify(code=1, result=f.read())
+  except:
+    return jsonify(code=0, result="Warning: Log file doesn't exist yet or empty log!!")
 
 
 def stopAllPartition():
@@ -256,12 +259,9 @@ def cloneRepository():
   return cloneRepo(data)
 
 
-def readFolder():
-  return getFolderContent(app.config, request.form['dir'])
-
-
-def openFolder():
-  return getFolder(app.config, request.form['dir'])
+def listDirectory():
+  folderList = listFolder(app.config, request.form['name'])
+  return jsonify(result=folderList)
 
 
 def createSoftware():
@@ -280,11 +280,6 @@ def setCurrentProject():
     return jsonify(code=0, result=("Can not setup this Software Release"))
 
 
-def manageProject():
-  return render_template('manageProject.html', workDir='workspace',
-                         project=getProjectList(app.config['workspace']))
-
-
 def getProjectStatus():
   path = realpath(app.config, request.form['project'])
   if path:
@@ -296,12 +291,17 @@ def getProjectStatus():
 #view for current software release files
 def editCurrentProject():
   project = os.path.join(app.config['etc_dir'], ".project")
-  if os.path.exists(project):
-    return render_template('softwareFolder.html', workDir='workspace',
+  projectList = listFolder(app.config, 'workspace')
+  if os.path.exists(project) and projectList:
+    return render_template('softwareFolder.html', workDir='runner_workdir',
                            project=open(project).read(),
-                           projectList=getProjectList(app.config['workspace']))
-  return redirect(url_for('configRepo'))
-
+                           projectList=projectList)
+  elif not projectList:
+    flash('Please clone slapos repository, or your own repository')
+    return redirect(url_for('manageRepository'))
+  else:
+    flash('Please, <br/>open or create a software to start with your project!!')
+    return redirect(url_for('openProject', method='open'))
 
 #create file or directory
 def createFile():
@@ -356,12 +356,13 @@ def getFileContent():
 
 
 def saveFileContent():
-  file_path = realpath(app.config, request.form['file'])
+  file_path = realpath(app.config, request.form['file'], False)
   if file_path:
-    open(file_path, 'w').write(request.form['content'].encode("utf-8"))
+    with open(file_path, 'w') as f:
+      f.write(request.form['content'].encode("utf-8"))
     return jsonify(code=1, result="")
   else:
-    return jsonify(code=0, result="Error: No such file!")
+    return jsonify(code=0, result="Rejected!! Cannot access to this location.")
 
 
 def changeBranch():
@@ -383,16 +384,13 @@ def newBranch():
     return jsonify(code=0, result="Can not read folder: Permission Denied")
 
 
-def getProjectDiff(project):
-  path = os.path.join(app.config['workspace'], project)
-  return render_template('projectDiff.html', project=project,
-                           diff=getDiff(path))
-def commitProjectFiles():
+def getProjectDiff():
   path = realpath(app.config, request.form['project'])
   if path:
-    return gitCommit(path, request.form['msg'])
+    return jsonify(code=1, result=getDiff(path))
   else:
-    return jsonify(code=0, result="Can not read folder: Permission Denied")
+    return jsonify(code=0,
+                  result="Error: No such file or directory. PERMISSION DENIED!")
 
 
 def pushProjectFiles():
@@ -452,6 +450,8 @@ def slapgridResult():
 
 
 def stopSlapgrid():
+  counter_file = os.path.join(app.config['runner_workdir'], '.turn-left')
+  open(counter_file, 'w+').write(str(0))
   result = killRunningProcess(request.form['type'])
   return jsonify(result=result)
 
@@ -461,7 +461,7 @@ def getPath():
   list = []
   for p in files:
     path = realpath(app.config, p)
-    if not p:
+    if not path:
       list = []
       break
     else:
@@ -527,6 +527,28 @@ def getParameterXml(request):
     return jsonify(code=1, result=parameters)
 
 
+#update user-defined slapgrid parameters
+def updateBuildAndRunConfig():
+  code = 1
+  try:
+    max_run_instance = int(request.form['max_run_instance'].strip())
+    max_run_software = int(request.form['max_run_software'].strip())
+  except ValueError:
+    code = 0
+    result = "Error! You should have provided an integer"
+  run_instance = (True if request.form['run_instance']=="true" else False)
+  run_software = (True if request.form['run_software']=="true" else False)
+  if code:
+    params =  {}
+    params['run_instance'] = run_instance
+    params['run_software'] = run_software
+    params['max_run_instance'] = max_run_instance
+    params['max_run_software'] = max_run_software
+    saveBuildAndRunParams(app.config, params)
+    result = "Your parameters have correctly been updated"
+  return jsonify(code=code, result=result)
+
+
 #update user account data
 def updateAccount():
   code = request.form['rcode'].strip()
@@ -562,7 +584,7 @@ def configAccount():
   account.append(request.form['name'].strip())
   code = request.form['rcode'].strip()
   recovery_code = open(os.path.join(app.config['etc_dir'], ".rcode"),
-                      "r").read()
+                      "r").read().strip()
   if code != recovery_code:
     return jsonify(code=0, result="Your password recovery code is not valid!")
   result = saveSession(app.config, account)
@@ -571,6 +593,18 @@ def configAccount():
   else:
     return jsonify(code=1, result="")
 
+def addUser():
+  code = request.form['rcode'].strip()
+  recovery_code = open(os.path.join(app.config['etc_dir'], ".rcode"),
+                      "r").read().strip()
+  if code != recovery_code:
+    return jsonify(code=0, result="Your password recovery code is not valid!")
+  if createNewUser(app.config, request.form['username'],
+                   request.form['password']):
+    return jsonify(code=1, result="New user succesfully saved")
+  else:
+    return jsonify(code=0, result="Problem while creating new user")
+
 
 #Global File Manager
 def fileBrowser():
@@ -630,6 +664,13 @@ def fileBrowser():
     elif opt == 17:
       #zip file
       result = file_request.unzipFile(dir, filename, newfilename)
+    elif opt == 20:
+      #Fancy Load folder
+      key = request.args.get('key')
+      dir = request.args.get('dir').encode('utf-8')
+      listfiles = request.args.get('listfiles', '')
+      data = file_request.fancylistDirs(dir, key, listfiles)
+      result = json.dumps(data)
     else:
       result = "ARGS PARSE ERROR: Bad option..."
   except Exception as e:
@@ -640,7 +681,7 @@ def fileBrowser():
 def editFile():
   return render_template('editFile.html', workDir='workspace',
     profile=urllib.unquote(request.args.get('profile', '')),
-    projectList=getProjectList(app.config['workspace']),
+    projectList=listFolder(app.config, 'workspace'),
     filename=urllib.unquote(request.args.get('filename', '')))
 
 def shell():
@@ -660,8 +701,6 @@ app.add_url_rule('/inspectSoftware', 'inspectSoftware', inspectSoftware)
 app.add_url_rule('/removeSoftware', 'removeSoftware', removeSoftware)
 app.add_url_rule('/runSoftwareProfile', 'runSoftwareProfile',
                  runSoftwareProfile, methods=['GET', 'POST'])
-app.add_url_rule('/viewSoftwareLog', 'viewSoftwareLog',
-                 viewSoftwareLog, methods=['GET'])
 app.add_url_rule('/editInstanceProfile', 'editInstanceProfile',
                  editInstanceProfile)
 app.add_url_rule('/inspectInstance', 'inspectInstance',
@@ -671,8 +710,10 @@ app.add_url_rule('/supervisordStatus', 'supervisordStatus',
 app.add_url_rule('/runInstanceProfile', 'runInstanceProfile',
                  runInstanceProfile, methods=['GET', 'POST'])
 app.add_url_rule('/removeInstance', 'removeInstance', removeInstance)
-app.add_url_rule('/viewInstanceLog', 'viewInstanceLog',
-                 viewInstanceLog, methods=['GET'])
+app.add_url_rule('/viewLog', 'viewLog',
+                 viewLog, methods=['GET'])
+app.add_url_rule("/getFileLog", 'getFileLog', getFileLog,
+                 methods=['POST'])
 app.add_url_rule('/stopAllPartition', 'stopAllPartition',
                  stopAllPartition, methods=['GET'])
 app.add_url_rule('/tailProcess/name/<process>', 'tailProcess',
@@ -691,12 +732,10 @@ app.add_url_rule("/checkFileType", 'checkFileType', checkFileType,
                  methods=['POST'])
 app.add_url_rule("/pullProjectFiles", 'pullProjectFiles', pullProjectFiles,
                  methods=['POST'])
-app.add_url_rule("/commitProjectFiles", 'commitProjectFiles', commitProjectFiles,
-                 methods=['POST'])
 app.add_url_rule("/pushProjectFiles", 'pushProjectFiles', pushProjectFiles,
                  methods=['POST'])
-app.add_url_rule("/getProjectDiff/<project>", 'getProjectDiff', getProjectDiff,
-                 methods=['GET'])
+app.add_url_rule("/getProjectDiff", 'getProjectDiff', getProjectDiff,
+                 methods=['POST'])
 app.add_url_rule("/newBranch", 'newBranch', newBranch, methods=['POST'])
 app.add_url_rule("/changeBranch", 'changeBranch', changeBranch, methods=['POST'])
 app.add_url_rule("/saveFileContent", 'saveFileContent', saveFileContent,
@@ -712,7 +751,6 @@ app.add_url_rule("/getProjectStatus", 'getProjectStatus', getProjectStatus,
                  methods=['POST'])
 app.add_url_rule('/openProject/<method>', 'openProject', openProject,
                  methods=['GET'])
-app.add_url_rule("/manageProject", 'manageProject', manageProject, methods=['GET'])
 app.add_url_rule("/setCurrentProject", 'setCurrentProject', setCurrentProject,
                  methods=['POST'])
 app.add_url_rule("/checkFolder", 'checkFolder', checkFolder, methods=['POST'])
@@ -720,17 +758,19 @@ app.add_url_rule('/createSoftware', 'createSoftware', createSoftware,
                  methods=['POST'])
 app.add_url_rule('/cloneRepository', 'cloneRepository', cloneRepository,
                  methods=['POST'])
-app.add_url_rule('/openFolder', 'openFolder', openFolder, methods=['POST'])
-app.add_url_rule('/readFolder', 'readFolder', readFolder, methods=['POST'])
-app.add_url_rule('/configRepo', 'configRepo', configRepo)
+app.add_url_rule('/listDirectory', 'listDirectory', listDirectory, methods=['POST'])
+app.add_url_rule('/manageRepository', 'manageRepository', manageRepository)
 app.add_url_rule("/saveParameterXml", 'saveParameterXml', saveParameterXml,
                  methods=['POST'])
 app.add_url_rule("/getPath", 'getPath', getPath, methods=['POST'])
 app.add_url_rule("/myAccount", 'myAccount', myAccount)
 app.add_url_rule("/updateAccount", 'updateAccount', updateAccount,
                  methods=['POST'])
+app.add_url_rule("/updateBuildAndRunConfig", 'updateBuildAndRunConfig', updateBuildAndRunConfig,
+                 methods=['POST'])
 app.add_url_rule("/fileBrowser", 'fileBrowser', fileBrowser,
                  methods=['GET', 'POST'])
 app.add_url_rule("/editFile", 'editFile', editFile, methods=['GET'])
 app.add_url_rule('/shell', 'shell', shell)
 app.add_url_rule('/isSRReady', 'isSRReady', isSRReady)
+app.add_url_rule('/addUser', 'addUser', addUser, methods=['POST'])
